@@ -1,38 +1,24 @@
 //! Contract handler
 //! The handler helps you deploy contract and test the contract you developing
 use std::cell::RefCell;
+use std::convert::TryInto;
 use std::fmt;
-use std::fs::{self, read};
-use std::path::PathBuf;
+use std::fs::read;
 use std::sync::Arc;
 
 use crate::errors::ContractError as Error;
 use crate::runtimes::traits::{VMMessageBuilder, VMResult, RT};
+use crate::types::Raw;
 
 use anyhow::{Context, Result};
-use ethereum_types::Address;
+use hex::decode;
 use serde_derive::{Deserialize, Serialize};
 
 #[derive(Clone, Deserialize, Serialize, Default)]
 pub struct ContractHandler {
-    /// The contract address, when the contract is not deployed,
-    /// the `connect` function called the contract will be automatically
-    /// deployed and the address field of the config will be filled
-    pub contract_address: Option<Address>,
-
-    /// The address of the sender, who calls the contract
-    #[serde(skip)]
-    pub sender_address: Address,
-
     /// The contract data in hex literal for eWasm binary, or a file path to
     /// the .ewasm file
     pub call_data: Option<String>,
-
-    /// If the contract_config_file_path pass from the connection string,
-    /// this field will be filled
-    #[serde(skip)]
-    pub config_file_path: Option<PathBuf>,
-
     #[serde(skip)]
     pub rt: Option<Arc<RefCell<dyn RT>>>,
 }
@@ -40,58 +26,45 @@ pub struct ContractHandler {
 impl fmt::Debug for ContractHandler {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("ContractHandler")
-            .field("contract_address", &self.contract_address)
-            .field("sender_address", &self.sender_address)
             .field("call_data", &self.call_data)
-            .field("config_file_path", &self.config_file_path)
             .field("rt", &self.rt.is_some())
             .finish()
     }
 }
 
 impl ContractHandler {
-    /// Reach the contract, if the contract is not exist, then deploy it first
-    pub fn connect(&mut self, gas: i64) -> Result<()> {
-        if self.contract_address.is_none() {
-            return self.deploy(gas);
-        }
-        Ok(())
-    }
-
-    /// deploy the contract from call data store contract address into config
-    fn deploy(&mut self, gas: i64) -> Result<()> {
-        if let Some(config_file_path) = self.config_file_path.take() {
-            if let Some(call_data) = self.call_data.take() {
-                let call_data = ContractHandler::get_call_data(call_data)?;
-                if call_data.len() < 4 {
-                    return Err(Error::ContractSizeError(call_data.len()).into());
-                }
-                if let Some(rt) = self.rt.take() {
-                    let msg = VMMessageBuilder {
-                        sender: Some(&self.sender_address),
-                        input_data: Some(&call_data),
-                        gas,
-                        ..Default::default()
-                    }
-                    .build()?;
-                    self.contract_address = Some(*rt.borrow_mut().deploy(msg)?);
-                    self.rt = Some(rt);
-                    fs::write(
-                        config_file_path,
-                        toml::to_string(self).expect("config generate error"),
-                    )
-                    .expect("config file can not be updated");
-                    return Ok(());
-                }
-                panic!("rt should be init when parsing the connection string")
+    /// run the call data as function directly, this function is suppose to be used for constructor
+    pub fn run_fn(
+        &mut self,
+        call_data: String,
+        input: Option<&[u8]>,
+        gas: i64,
+    ) -> Result<VMResult> {
+        if let Some(rt) = self.rt.take() {
+            let call_data = ContractHandler::get_call_data(call_data)?;
+            let mut input_data: Vec<u8> = Vec::new();
+            if let Some(input) = input {
+                input_data.extend_from_slice(input);
             }
-            return Err(Error::InsufficientContractInfoError.into());
+            let sender = Raw::default();
+            let msg = VMMessageBuilder {
+                sender: Some(&sender),
+                input_data: Some(&input_data),
+                gas,
+                code: Some(&call_data),
+                ..Default::default()
+            }
+            .build()?;
+            let result = Ok(rt.borrow_mut().execute(msg)?);
+            self.rt = Some(rt);
+            return result;
         }
-        panic!("config_file_path should be update when ContractHandler constructed")
+        panic!("rt should be init when parsing the connection string")
     }
 
     pub fn execute(
         &mut self,
+        addr: Option<&str>,
         fun_sig: [u8; 4],
         input: Option<&[u8]>,
         gas: i64,
@@ -105,8 +78,18 @@ impl ContractHandler {
                     input_data.extend_from_slice(input);
                 }
 
+                let sender = if let Some(addr) = addr {
+                    let byte20: [u8; 20] = decode(addr)
+                        .expect("address should be hex format")
+                        .try_into()
+                        .expect("address should be byte20");
+                    Raw::from_raw_address(&byte20)
+                } else {
+                    Raw::default()
+                };
+
                 let msg = VMMessageBuilder {
-                    sender: Some(&self.sender_address),
+                    sender: Some(&sender),
                     input_data: Some(&input_data),
                     gas,
                     code: Some(&call_data),
